package de.mcs.tools.sps.emulator.resources;

import java.io.PrintWriter;
import java.io.StringWriter;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response.Status;

import de.mcs.tools.sps.emulator.Emulator;
import de.mcs.tools.sps.emulator.EmulatorFactory;
import de.mcs.tools.sps.emulator.model.CommandModel.COMMAND;
import de.mcs.tools.sps.emulator.model.WebSessionModel;
import de.mcs.tools.sps.emulator.model.WorkingModel.WORKINGSTATE;

@Path("/emulator")
@Produces(MediaType.APPLICATION_JSON)
public class EmulatorResource {

  @GET
  public WebSessionModel newSession() {
    return new WebSessionModel();
  }

  @POST
  public WebSessionModel getSession(WebSessionModel model) {
    model = checkModel(model);
    Emulator emulator = EmulatorFactory.getEmulator(model);
    COMMAND actualCommand = model.getCommand().getActualCommand();
    try {
      switch (actualCommand) {
      case NN:
        break;
      case START:
        emulator.start();
        break;
      case NEXT:
        emulator.next();
        break;
      case STOP:
        emulator.stop();
        break;
      case COMPILE:
        emulator.compile();
        break;
      default:
        break;
      }
    } catch (Exception e) {
      String stackTrace = (e != null) ? "\n" + getStacktraceAsString(e) : "";
      String errorMessage = String.format("%s%s", e.getClass().getName(),
          e.getMessage() == null ? "" : ":" + e.getMessage());
      model.setMessage(errorMessage);
      model.setStacktrace(stackTrace);
      model.setError(true);
      e.printStackTrace();
    }
    if (model.isError()) {
      throw new WebApplicationException(model.getMessage(), Status.BAD_REQUEST);
    }
    return model;
  }

  private WebSessionModel checkModel(WebSessionModel model) {
    model.getProgram().rebuildHash();
    COMMAND actualCommand = model.getCommand().getActualCommand();
    model.getCommand().evaluateAvailableCommands(actualCommand);
    if (model.getProgram().isModified() && WORKINGSTATE.EMULATE.equals(model.getWork().getWorkingstate())) {
      model.setError(false);
      model.setMessage("emulator active: can't changed the source.");
    }
    return model;
  }

  private String getStacktraceAsString(Throwable e) {
    String stackTrace = "";
    if (e != null) {
      StringWriter writer = new StringWriter();
      e.printStackTrace(new PrintWriter(writer));
      stackTrace += writer.toString();
    }
    return stackTrace;
  }

}
