package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestBLDA {

  private static final int BYTE_VALUE_ADC = 0xf0;
  private final String MNEMONIC = "BLDA";

  private static final int BYTE_VALUE_RC = 0xf2;

  @Test
  void testMnemonicRC() throws SyntaxError {
    BLDA mnemonic = new BLDA(MNEMONIC + " RC1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_RC, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new BLDA(MNEMONIC + " RC2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_RC + 1, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      BLDA mno1 = new BLDA(MNEMONIC + " RC3");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      BLDA mno1 = new BLDA(MNEMONIC + " RC0");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicADC() throws SyntaxError {
    BLDA mnemonic = new BLDA(MNEMONIC + " ADC1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_ADC, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new BLDA(MNEMONIC + " ADC2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_ADC + 1, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      BLDA mno1 = new BLDA(MNEMONIC + " ADC3");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      BLDA mno1 = new BLDA(MNEMONIC + " ADC0");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " ADC1", 0);
    assertEquals(BLDA.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 5", 0);
    });
  }
}
