package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

class TestSTA {

  private static final int BYTE_VALUE_DOUT = 0x54;
  private static final int BYTE_VALUE_PWM = 0x59;
  private static final int BYTE_VALUE_SRV = 0x5b;
  private final String MNEMONIC = "STA";

  @Test
  void testMnemonicDOUT() throws SyntaxError {
    STA mnemonic = new STA(MNEMONIC + " DOUT");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_DOUT, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new STA(MNEMONIC + " DOUT1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_DOUT + 1, mnemonic.getByte());

    mnemonic = new STA(MNEMONIC + " DOUT4");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_DOUT + 0x04, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      STA mno1 = new STA(MNEMONIC + " DOUT5");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicSRV() throws SyntaxError {
    STA mnemonic = new STA(MNEMONIC + " SRV1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_SRV, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new STA(MNEMONIC + " SRV2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_SRV + 1, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      STA mno1 = new STA(MNEMONIC + " SRV3");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      STA mno1 = new STA(MNEMONIC + " SRV0");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicPWM() throws SyntaxError {
    STA mnemonic = new STA(MNEMONIC + " PWM1");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_PWM, mnemonic.getByte());
    assertTrue(mnemonic.hasArgument());
    assertFalse(mnemonic.isUsingPage());

    mnemonic = new STA(MNEMONIC + " PWM2");
    mnemonic.checkArgument();
    assertEquals(BYTE_VALUE_PWM + 1, mnemonic.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      STA mno1 = new STA(MNEMONIC + " PWM3");
      mno1.checkArgument();
    });

    Assertions.assertThrows(IllegalArgument.class, () -> {
      STA mno1 = new STA(MNEMONIC + " PWM0");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " SRV1", 0);
    assertEquals(STA.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic(MNEMONIC + " DOUT", 0);
    assertEquals(STA.class, mnemonic.getClass());

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC, 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 0x00", 0);
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      MnemonicFactory.getMnemonic(MNEMONIC + " 5", 0);
    });
  }
}
