/**
 * MCS Media Computer Software
 * Copyright 2018 by Wilfried Klaas
 * Project: SPSEmulator
 * File: Macro.java
 * EMail: W.Klaas@gmx.de
 * Created: 06.12.2018 wklaa_000
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
 */
package de.mcs.tools.sps;

import java.util.ArrayList;
import java.util.List;

import de.mcs.tools.sps.exceptions.IllegalArgument;
import de.mcs.tools.sps.exceptions.SyntaxError;

/**
 * @author wklaa_000
 *
 */
public class Macro {

  private String[] args;
  private String name;
  private List<String> lines;

  public Macro(String name, String[] args) {
    this.name = name;
    this.args = args;
    lines = new ArrayList<>();
  }

  @Override
  public String toString() {
    StringBuilder b = new StringBuilder();
    b.append(String.format("name: %s", name));
    if (args != null) {
      StringBuilder b1 = new StringBuilder();
      b1.append("[");
      boolean start = true;
      for (String arg : args) {
        if (!start) {
          b1.append(",");
        }
        start = false;
        b1.append(arg);
      }
      b1.append("]");
      b.append(String.format(", args: %s", b1));
    }
    return b.toString();
  }

  public String getName() {
    return name;
  }

  public void addLine(String line) {
    lines.add(line);
  }

  public List<String> processMacro(String[] values, int lineNumber) throws SyntaxError {
    if ((values != null) && (args == null)) {
      throw new IllegalArgument(lineNumber,
          String.format("count of macro arguments (%d) are not the same as in macro definition (%d) of macro \"%s\".",
              values.length, 0, name));
    }
    if ((values == null) && (args != null)) {
      throw new IllegalArgument(lineNumber,
          String.format("count of macro arguments (%d) are not the same as in macro definition (%d) of macro \"%s\".",
              0, args.length, name));
    }
    if (values != null && values.length != args.length) {
      throw new SyntaxError(lineNumber,
          String.format("count of macro arguments (%d) are not the same as in macro definition (%d) of macro \"%s\".",
              values.length, args.length, name));
    }
    List<String> mnemonics = new ArrayList<>();
    lines.forEach(l -> {
      String line = l;
      if (args != null) {
        for (int i = 0; i < args.length; i++) {
          String key = args[i];
          String value = values[i];
          line = line.replace(key, value);
        }
      }
      mnemonics.add(line);
    });
    return mnemonics;
  }
}
