/**
 * 
 */
package de.mcs.tools.sps.service;

import java.io.ByteArrayOutputStream;
import java.util.List;

import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;

import com.codahale.metrics.annotation.Timed;

import de.mcs.tools.sps.IntelHEXOutputter;
import de.mcs.tools.sps.Outputter;
import de.mcs.tools.sps.SPSAssembler;
import de.mcs.tools.sps.exceptions.HardwareException;
import de.mcs.tools.sps.mnemonic.HARDWARE;
import de.mcs.tools.sps.mnemonic.Mnemonic;
import de.mcs.tools.sps.service.model.AsmModel;

/**
 * @author w.klaas
 *
 */
@Path("/assemble")
@Produces(MediaType.APPLICATION_JSON)
public class AsmResource {

  @POST
  @Timed
  public AsmModel assemble(AsmModel model) throws HardwareException {
    try {
      HARDWARE destination = HARDWARE.HOLTEK;

      try {
        destination = HARDWARE.valueOf((model.getHardware().toUpperCase()));
      } catch (IllegalArgumentException e) {
        throw new HardwareException(String.format("Hardware %s unknown for this assembler", model.getHardware()));
      }

      SPSAssembler spsAssembler = new SPSAssembler();
      spsAssembler.setDestination(destination);
      spsAssembler.doWork(model.getContent());

      List<Mnemonic> mnemonics = spsAssembler.getMnemonics();
      ByteArrayOutputStream output = new ByteArrayOutputStream();

      Outputter outputter = new IntelHEXOutputter();
      outputter.output(mnemonics, output);

      model.setHexfile(new String(output.toByteArray()));

      return model;
    } catch (Exception e) {
      throw new WebApplicationException(e.getMessage(), e);
    }
  }

}
